# coding: utf-8
from __future__ import absolute_import, unicode_literals

__title__ = 'djangorestframework-sso'
__version__ = '0.0.8'
__author__ = 'Lenno Nagel'
__license__ = 'MIT'
__copyright__ = 'Copyright 2016 Namespace OÜ'

# Version synonym
VERSION = __version__
